@@ -4,7 +4,6 @@ import inspect
 import logging
 import sys
 import time
-import types
 
 import progressbar
 import six
@@ -20,7 +19,7 @@ config.register('logging_level',
 
 
 class StatusLogger(object):
-    '''
+    """
     StatusLogger is a wrapper around `logging.Logger` that allows for consistent
     treatment of logging messages. While not strictly required,
     it simplifies and abstracts the usage of the logging module within omniduct.
@@ -33,7 +32,7 @@ class StatusLogger(object):
 
     StatusLogger will automatically detect the context of the logged message, and
     include it in the log messages.
-    '''
+    """
 
     def __init__(self, auto_scoping=False):
         self.__scopes = []
@@ -49,6 +48,14 @@ class StatusLogger(object):
 
         self._progress_bar = None
 
+    @property
+    def disabled(self):
+        return self.__get_logger_instance().disabled
+
+    @disabled.setter
+    def disabled(self, disabled):
+        self.__get_logger_instance().disabled = disabled
+
     def _scope_enter(self, name, timed=False, extra=None):
         if config.logging_level < logging.INFO:
             print("\t" * len(self.__scopes) + "Entering manual scope: {}".format(name), file=sys.stderr)
@@ -77,7 +84,7 @@ class StatusLogger(object):
             print("\t" * len(self.__scopes) + "Exited manual scope: {}".format(scope['name']), file=sys.stderr)
         elif 'has_logged' in scope:
             if len(self.__scopes) != 0:
-                self.current_scope_props['has_logged'] = self.current_scope_props.get('has_logged') or props['has_logged']
+                self.current_scope_props['has_logged'] = self.current_scope_props.get('has_logged') or props.get('has_logged', False)
 
     def __get_time(self, seconds):
         m, s = divmod(seconds, 60)
@@ -97,41 +104,42 @@ class StatusLogger(object):
 
     @property
     def current_scopes(self):
-        '''
+        """
         The current logger scopes. This is not designed to work with multiple
         threads.
-        '''
+        """
         return detect_scopes()
 
     @property
     def current_scope_props(self):
-        ''''
+        """
         The properties for the most nested manual scope.
-        '''
+        """
         if len(self.__scopes) == 0:
             return None
         return self.__scopes[-1]
 
-    def __get_progress_bar(self):
+    def __get_progress_bar(self, indeterminate=False):
         if self._progress_bar is None:
             if config.logging_level >= logging.INFO:
                 prefix = ": ".join(self.current_scopes) + ": "
             else:
                 prefix = "\t" * len(self.current_scopes)
-            self._progress_bar = progressbar.ProgressBar(widgets=[prefix, progressbar.widgets.Bar()],
+            self._progress_bar = progressbar.ProgressBar(widgets=[prefix, progressbar.widgets.RotatingMarker() if indeterminate else progressbar.widgets.Bar(), progressbar.widgets.Timer(format=' %(elapsed)s')],
                                                          redirect_stderr=True,
-                                                         redirect_stdout=True).start()
+                                                         redirect_stdout=True,
+                                                         max_value=100).start()
 
         return self._progress_bar
 
-    def progress(self, progress, complete=False):
-        '''
+    def progress(self, progress=None, complete=False, indeterminate=False):
+        """
         Set the current progress to `progress`, and if not already showing, display
         a progress bar. If `complete` evaluates to True, then finish displaying the progress.
-        '''
+        """
         complete = complete or (self.current_scope_props is None)  # Only leave progress bar open if within a scope
         if config.logging_level <= logging.INFO:
-            self.__get_progress_bar().update(progress)
+            self.__get_progress_bar(indeterminate=indeterminate).update(progress)
             if complete:
                 self.__get_progress_bar().finish(end=None)
                 self._progress_bar = None
@@ -139,31 +147,33 @@ class StatusLogger(object):
     # Logging emulation
 
     def __get_logger_instance(self, context=None):
-        '''
+        """
         Get a `logger.Logger` instance for the provided context; inferring the
         context from the runtime stack if the provided context is `None`.
-        '''
+        """
         if context is None:
             try:
                 caller = inspect.stack()[2]
                 context = inspect.getmodule(caller.frame).__name__
             except:
                 context = 'omniduct'
+        if not context == 'omniduct' and not context.startswith('omniduct.'):
+            context = 'omniduct.external.{}'.format(context)
         return logging.getLogger(context)
 
     def __getattr__(self, name):
-        '''
+        """
         Return the attributes of the wrapped `logging.Logger` instance rather
         than this one (unless the property actually exists in StatusLogger).
-        '''
+        """
         return getattr(self.__get_logger_instance(), name)
 
     def setLevel(self, level, context=None):
-        '''
+        """
         Add a keyword argument `context` to the standard `setLevel` method, in
         order to allow for fine-grained logging levels, while retaining the
         simplicity of a single "logger" instance.
-        '''
+        """
         self.__get_logger_instance(context).setLevel(level)
 
 
@@ -185,15 +195,18 @@ def detect_scopes():
     seen = set()
     for scope in scopes[::-1]:
         if scope not in seen:
-            out_scopes.append(scope if isinstance(scope, six.string_types) else (scope.name or scope.__class__.__name__))
+            out_scopes.append(
+                scope
+                if isinstance(scope, six.string_types) else
+                (getattr(scope, "LOGGING_SCOPE", None) or getattr(scope, "name", None) or scope.__class__.__name__))
             seen.add(scope)
     return out_scopes
 
 
 class LoggingHandler(logging.Handler):
-    '''
+    """
     An implementation of Logging.Handler to render the logging methods shown in Omniduct and derivatives.
-    '''
+    """
 
     def __init__(self, level=logging.NOTSET):
         logging.Handler.__init__(self, level=level)
@@ -245,12 +258,12 @@ class LoggingHandler(logging.Handler):
 
 
 def logging_scope(name, *wargs, **wkwargs):
-    '''
+    """
     A decorator to add the decorated function as a new logging scope, with name `name`.
     All additional arguments are passed to `StatusLogger._scope_enter`. Current
     supported keyword arguments are "timed", in which case when the scope closes,
     the duration of the call is shown.
-    '''
+    """
     def logging_scope(func, *args, **kwargs):
         logger._scope_enter(name, *wargs, **wkwargs)
         success = True
